/* asm41 - HP-41 assembler */

/* Copyright (C) 2014 Kari Pasanen */
/* Full copyright notice in document file asm41.html */

/* Definitions of asm41 language */

struct descriptor {
        int instr;
        int param;
        char mnem[8];
        enum {
                PARAM_TYPE_CONST,
                PARAM_TYPE_IMM_WORD,
                PARAM_TYPE_NONE,
                PARAM_TYPE_D,
                PARAM_TYPE_N,
                PARAM_TYPE_TEF,
                PARAM_TYPE_REL,
                PARAM_TYPE_ABS,
                PARAM_TYPE_DCONST
        } param_type;
};

static const struct descriptor CON_descriptor[] = {
        {   -1,  -1, "CON",     PARAM_TYPE_CONST    },
        {    0,  -1, "DCON",    PARAM_TYPE_DCONST   },
        {    2,  -1, "UCCON",   PARAM_TYPE_DCONST   }
};

static const struct descriptor class0_descriptor[] = {
        {  0x0, 0x0, "NOP",     PARAM_TYPE_NONE     },
        {  0x0, 0x1, "WROM",    PARAM_TYPE_NONE     },
        {  0x0, 0x2, "CON",     PARAM_TYPE_CONST    },
        {  0x0, 0x3, "CON",     PARAM_TYPE_CONST    },
        {  0x0, 0x4, "ENBANK1", PARAM_TYPE_NONE     },
        {  0x0, 0x5, "ENBANK3", PARAM_TYPE_NONE     },
        {  0x0, 0x6, "ENBANK2", PARAM_TYPE_NONE     },
        {  0x0, 0x7, "ENBANK4", PARAM_TYPE_NONE     },
        {  0x0, 0x8, "HPIL0=C", PARAM_TYPE_NONE     },
        {  0x0, 0x9, "HPIL1=C", PARAM_TYPE_NONE     },
        {  0x0, 0xA, "HPIL2=C", PARAM_TYPE_NONE     },
        {  0x0, 0xB, "HPIL3=C", PARAM_TYPE_NONE     },
        {  0x0, 0xC, "HPIL4=C", PARAM_TYPE_NONE     },
        {  0x0, 0xD, "HPIL5=C", PARAM_TYPE_NONE     },
        {  0x0, 0xE, "HPIL6=C", PARAM_TYPE_NONE     },
        {  0x0, 0xF, "HPIL7=C", PARAM_TYPE_NONE     },
        {  0x1, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0x1, 0xF, "ST=0",    PARAM_TYPE_NONE     },
        {  0x1,  -1, "CF",      PARAM_TYPE_D        },
        {  0x2, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0x2, 0xF, "CLRKEY",  PARAM_TYPE_NONE     },
        {  0x2,  -1, "SF",      PARAM_TYPE_D        },
        {  0x3, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0x3, 0xF, "?KEY",    PARAM_TYPE_NONE     },
        {  0x3,  -1, "?FS",     PARAM_TYPE_D        },
        {  0x4,  -1, "LC",      PARAM_TYPE_N        },
        {  0x5, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0x5, 0xF, "DECPT",   PARAM_TYPE_NONE     },
        {  0x5,  -1, "?PT=",    PARAM_TYPE_D        },
        {  0x6, 0x1, "G=C",     PARAM_TYPE_NONE     },
        {  0x6, 0x2, "C=G",     PARAM_TYPE_NONE     },
        {  0x6, 0x3, "C<>G",    PARAM_TYPE_NONE     },
        {  0x6, 0x5, "M=C",     PARAM_TYPE_NONE     },
        {  0x6, 0x6, "C=M",     PARAM_TYPE_NONE     },
        {  0x6, 0x7, "C<>M",    PARAM_TYPE_NONE     },
        {  0x6, 0x9, "F=ST",    PARAM_TYPE_NONE     },
        {  0x6, 0xA, "ST=F",    PARAM_TYPE_NONE     },
        {  0x6, 0xB, "ST<>F",   PARAM_TYPE_NONE     },
        {  0x6, 0xD, "ST=C",    PARAM_TYPE_NONE     },
        {  0x6, 0xE, "C=ST",    PARAM_TYPE_NONE     },
        {  0x6, 0xF, "C<>ST",   PARAM_TYPE_NONE     },
        {  0x6,  -1, "CON",     PARAM_TYPE_CONST    },
        {  0x7, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0x7, 0xF, "INCPT",   PARAM_TYPE_NONE     },
        {  0x7,  -1, "PT=",     PARAM_TYPE_D        },
        {  0x8, 0x0, "FORGET",  PARAM_TYPE_NONE     },
        {  0x8, 0x1, "POWOFF",  PARAM_TYPE_NONE     },
        {  0x8, 0x2, "SELP",    PARAM_TYPE_NONE     },
        {  0x8, 0x3, "SELQ",    PARAM_TYPE_NONE     },
        {  0x8, 0x4, "?P=Q",    PARAM_TYPE_NONE     },
        {  0x8, 0x5, "?LOWBAT", PARAM_TYPE_NONE     },
        {  0x8, 0x6, "CLRABC",  PARAM_TYPE_NONE     },
        {  0x8, 0x7, "GOTOC",   PARAM_TYPE_NONE     },
        {  0x8, 0x8, "C=KEY",   PARAM_TYPE_NONE     },
        {  0x8, 0x9, "SETHEX",  PARAM_TYPE_NONE     },
        {  0x8, 0xA, "SETDEC",  PARAM_TYPE_NONE     },
        {  0x8, 0xB, "DISOFF",  PARAM_TYPE_NONE     },
        {  0x8, 0xC, "DISTOG",  PARAM_TYPE_NONE     },
        {  0x8, 0xD, "RTNC",    PARAM_TYPE_NONE     },
        {  0x8, 0xE, "RTNNC",   PARAM_TYPE_NONE     },
        {  0x8, 0xF, "RTN",     PARAM_TYPE_NONE     },
        {  0x9,  -1, "PERTCT",  PARAM_TYPE_N        },
        {  0xA,  -1, "WRIT",    PARAM_TYPE_N        },
        {  0xB, 0x7, "CON",     PARAM_TYPE_CONST    },
        {  0xB, 0xF, "CON",     PARAM_TYPE_CONST    },
        {  0xB,  -1, "?PF",     PARAM_TYPE_D        },
        {  0xC, 0x1, "N=C",     PARAM_TYPE_NONE     },
        {  0xC, 0x2, "C=N",     PARAM_TYPE_NONE     },
        {  0xC, 0x3, "C<>N",    PARAM_TYPE_NONE     },
        {  0xC, 0x4, "LDI",     PARAM_TYPE_IMM_WORD },
        {  0xC, 0x5, "PUSH",    PARAM_TYPE_NONE     },
        {  0xC, 0x6, "POP",     PARAM_TYPE_NONE     },
        {  0xC, 0x8, "GOKEY",   PARAM_TYPE_NONE     },
        {  0xC, 0x9, "RAMSEL",  PARAM_TYPE_NONE     },
        {  0xC, 0xB, "WDATA",   PARAM_TYPE_NONE     },
        {  0xC, 0xC, "RROM",    PARAM_TYPE_NONE     },
        {  0xC, 0xD, "C=C|A",   PARAM_TYPE_NONE     },
        {  0xC, 0xE, "C=C&A",   PARAM_TYPE_NONE     },
        {  0xC, 0xF, "PERSEL",  PARAM_TYPE_NONE     },
        {  0xC,  -1, "CON",     PARAM_TYPE_CONST    },
        {  0xD,  -1, "CON",     PARAM_TYPE_CONST    },
        {  0xE, 0x0, "RDATA",   PARAM_TYPE_NONE     },
        {  0xE,  -1, "READ",    PARAM_TYPE_N        },
        {  0xF, 0x7, "WCMD",    PARAM_TYPE_NONE     },
        {  0xF, 0xF, "CON",     PARAM_TYPE_CONST    },
        {  0xF,  -1, "RCR",     PARAM_TYPE_D        }
};

static const struct descriptor class1_descriptor[] = {
        {  0x0,  -1, "XQNC",    PARAM_TYPE_ABS      },
        {  0x1,  -1, "XQC",     PARAM_TYPE_ABS      },
        {  0x2,  -1, "GONC",    PARAM_TYPE_ABS      },
        {  0x3,  -1, "GOC",     PARAM_TYPE_ABS      }
};

static const struct descriptor class2_descriptor[] = {
        { 0x00,  -1, "A=0",     PARAM_TYPE_TEF      },
        { 0x01,  -1, "B=0",     PARAM_TYPE_TEF      },
        { 0x02,  -1, "C=0",     PARAM_TYPE_TEF      },
        { 0x03,  -1, "A<>B",    PARAM_TYPE_TEF      },
        { 0x04,  -1, "B=A",     PARAM_TYPE_TEF      },
        { 0x05,  -1, "A<>C",    PARAM_TYPE_TEF      },
        { 0x06,  -1, "C=B",     PARAM_TYPE_TEF      },
        { 0x07,  -1, "B<>C",    PARAM_TYPE_TEF      },
        { 0x08,  -1, "A=C",     PARAM_TYPE_TEF      },
        { 0x09,  -1, "A=A+B",   PARAM_TYPE_TEF      },
        { 0x0A,  -1, "A=A+C",   PARAM_TYPE_TEF      },
        { 0x0B,  -1, "A=A+1",   PARAM_TYPE_TEF      },
        { 0x0C,  -1, "A=A-B",   PARAM_TYPE_TEF      },
        { 0x0D,  -1, "A=A-1",   PARAM_TYPE_TEF      },
        { 0x0E,  -1, "A=A-C",   PARAM_TYPE_TEF      },
        { 0x0F,  -1, "C=C+C",   PARAM_TYPE_TEF      },
        { 0x10,  -1, "C=A+C",   PARAM_TYPE_TEF      },
        { 0x11,  -1, "C=C+1",   PARAM_TYPE_TEF      },
        { 0x12,  -1, "C=A-C",   PARAM_TYPE_TEF      },
        { 0x13,  -1, "C=C-1",   PARAM_TYPE_TEF      },
        { 0x14,  -1, "C=-C",    PARAM_TYPE_TEF      },
        { 0x15,  -1, "C=-C-1",  PARAM_TYPE_TEF      },
        { 0x16,  -1, "?B#0",    PARAM_TYPE_TEF      },
        { 0x17,  -1, "?C#0",    PARAM_TYPE_TEF      },
        { 0x18,  -1, "?A<C",    PARAM_TYPE_TEF      },
        { 0x19,  -1, "?A<B",    PARAM_TYPE_TEF      },
        { 0x1A,  -1, "?A#0",    PARAM_TYPE_TEF      },
        { 0x1B,  -1, "?A#C",    PARAM_TYPE_TEF      },
        { 0x1C,  -1, "ASR",     PARAM_TYPE_TEF      },
        { 0x1D,  -1, "BSR",     PARAM_TYPE_TEF      },
        { 0x1E,  -1, "CSR",     PARAM_TYPE_TEF      },
        { 0x1F,  -1, "ASL",     PARAM_TYPE_TEF      }
};

static const struct descriptor class3_descriptor[] = {
        {  0x0,  -1, "JNC",     PARAM_TYPE_REL      },
        {  0x1,  -1, "JC",      PARAM_TYPE_REL      }
};

static const char *D_param[] = {
        "3", "4", "5", "10", "8", "6", "11", "14",
        "2", "9", "7", "13", "1", "12", "0", "15"
};

static const char *N_param[] = {
        "0", "1", "2", "3", "4", "5", "6", "7",
        "8", "9", "A", "B", "C", "D", "E", "F"
};

static const char *TEF_param[] = {
        "PT", "X", "WPT", "ALL", "PQ", "XS", "M", "S"
};
